package Competitor;

# Competitors have the following fields:
#     LNAME: last name
#     FNAME: first name
#     SCHOOL: the school.  Note that there is no difference between
# clicking on "MIT" and typing that in.
#     EMAIL: email address
#     EMAIL_EXISTS: this is a 1/0 flag, set if "No Email Address" is
# clicked on the starting page.
#     NUMBER: their competitor number
#     TBA: a 1/0 flag indicating if this Competitor object is a TBA placeholder

# FAST_COMPARE is obsolecent: it was an attempt to make comparing
# competitors faster, but didn't work.

sub new {
  my $proto=shift;
  my $class=ref($proto) || $proto;
  my $self={};
  my($lname, $fname, $school, $email, $email_exists)=@_;

# Check last name integrity and throw out bad names.
  $_=$lname;
# We allow '-' and ',' for "Ismail-Beigi" and "Thomas Nugent, Jr"
  if (/[^a-zA-Z -,]/) {
    Error::BadNameSpecification($_);
  }
  if ($lname eq "") {
    Error::BlankNameSpecification(0);
  }

# Check first name integrity and throw out bad names.
  $_=$fname;
  if (/[^a-zA-Z ]/) {
    Error::BadNameSpecification($_);
  }
  if ($fname eq "") {
    Error::BlankNameSpecification(1);
  }

# Upshift the first letter of the last name, unless it's a compound
# name, like von Deck, in which case, we upshift the second word.
  if ($lname =~ /^(de|von|van)/) {
    $lname=~s/^(de|von|van) ([a-z])/$1 \u$2/;
  } else {
    $lname=~s/^([a-z])/\u$1/;
  }

# Upshift the first letter of the first name.
  $fname=~s/^([a-z])/\u$1/;

# Usernames need to be of the format aoeu@aoeu.aoeu.aoeu.aoeu.aoeu, etc
  if ($email_exists) {
    $_=$email;
    if (!(/\w+\@\w+\.(\w+\.*)/)) {
      Error::BadEmailAddress($email);
    }
  }

# Blank schools are bad.  No School Affiliation is a school.  :)
  if ($school eq "") {
    Error::BlankSchoolSpecification();
  }

  $self->{LNAME}=$lname;
  $self->{FNAME}=$fname;
  $self->{SCHOOL}=$school;
  $self->{EMAIL}=$email;
  $self->{EMAIL_EXISTS}=$email_exists;
  $self->{NUMBER}=-1;
  $self->{TBA}=0;

  $self->{FAST_COMPARE}=$lname;
#  $self->{F_PARTNERS}=null;
#  $self->{L_PARTNERS}=null;
  
  bless($self, $class);
  return($self);
}

sub newFromDB {
  my $proto=shift;
  my $class=ref($proto) || $proto;
  my $IN=$_[0];
  my $self={};
  
  $_=<$IN>;  chop;
  $self->{LNAME}=$_;
  $self->{FAST_COMPARE}=$_;
  $_=<$IN>;  chop;
  $self->{FNAME}=$_;
  $_=<$IN>;  chop;
  $self->{SCHOOL}=$_;
  $_=<$IN>;  chop;
  $self->{EMAIL}=$_;
  $_=<$IN>;  chop;
  $self->{EMAIL_EXISTS}=$_;
  $_=<$IN>;  chop;
  $self->{NUMBER}=$_;
  $_=<$IN>;  chop;
  s/true/1/;
  s/false/0/;
  $self->{TBA}=$_;
  
#  $_=shift(@$IN);  chop;
#  $self->{LNAME}=$_;
#  $_=shift(@$IN);  chop;
#  $self->{FNAME}=$_;
#  $_=shift(@$IN);  chop;
#  $self->{SCHOOL}=$_;
#  $_=shift(@$IN);  chop;
#  $self->{EMAIL}=$_;
#  $_=shift(@$IN);  chop;
#  $self->{EMAIL_EXISTS}=$_;
#  $_=shift(@$IN);  chop;
#  $self->{NUMBER}=$_;
#  $_=shift(@$IN);  chop;
#  $self->{TBA}=$_;
  
  bless($self, $class);
  return($self);
}

sub new2 {
  my $proto=shift;
  my $class=ref($proto) || $proto;
  my $self={};
  
  $self->{LNAME}=$_[0];
  $self->{FAST_COMPARE}=$_[0];
  $self->{FNAME}=$_[1];
  $self->{SCHOOL}=$_[2];
  $self->{EMAIL}=$_[3];
  $self->{EMAIL_EXISTS}=$_[4];
  $self->{NUMBER}=$_[5];
  $_=$_[6];
  s/true/1/;
  s/false/0/;
  $self->{TBA}=$_;
  
  bless($self, $class);
  return($self);
}

sub newTBA {
  my $proto=shift;
  my $class=ref($proto) || $proto;
  my $self={};
  
  $self->{LNAME}="TBA";
  $self->{FNAME}="";
  $self->{SCHOOL}="";
  $self->{EMAIL}="";
  $self->{EMAIL_EXISTS}=0;
  $self->{NUMBER}=-1;
  $self->{TBA}=1;
  $self->{FAST_COMPARE}="";
  
  bless($self, $class);
  return($self);
}

sub assemble_competitors {
  my $post_data_ref=$_[0];
# $post_data is an assoc array.

  my $leader;
  my $follower;
  my $email_exists;
  my $school;

# Given a POST stream, read from it and assemble the two Competitors passed in.

  # leader
  if ($$post_data_ref{"leaderTBA"} eq "true") {
#    logmsg("assemble_competitors: leader is TBA");
    $leader=Competitor->newTBA();
  } else {
    if ($$post_data_ref{"leader_email_exists"} eq "false") {
      $email_exists=0;
    } else {
      $email_exists=1;
    }
# $school is the checkbox...
    $school=$$post_data_ref{"leader_school"};
# ...so if the checkbox is blank, look at the Fill In The Blank field.
    if ($school eq "") {
      $leader=Competitor->new($$post_data_ref{"leader_lname"},
			      $$post_data_ref{"leader_fname"},
			      $$post_data_ref{"leader_otherschool"},
			      $$post_data_ref{"leader_email"},
			      $email_exists);
   } else {
      $leader=Competitor->new($$post_data_ref{"leader_lname"},
			      $$post_data_ref{"leader_fname"},
			      $school,
			      $$post_data_ref{"leader_email"},
			      $email_exists);
    }
  }

# Rinse wash lather...
  # follower
  if ($$post_data_ref{"followerTBA"} eq "true") {
    $follower=Competitor->newTBA();
  } else {
    if ($$post_data_ref{"follower_email_exists"} eq "false") {
      $email_exists=0;
    } else {
      $email_exists=1;
    }
    $school=$$post_data_ref{"follower_school"};
    if ($school eq "") {
      $follower=Competitor->new($$post_data_ref{"follower_lname"},
			      $$post_data_ref{"follower_fname"},
			      $$post_data_ref{"follower_otherschool"},
			      $$post_data_ref{"follower_email"},
			      $email_exists);
    } else {
      $follower=Competitor->new($$post_data_ref{"follower_lname"},
			      $$post_data_ref{"follower_fname"},
			      $school,
			      $$post_data_ref{"follower_email"},
			      $email_exists);
    }
  }

  if ($leader->isTBA() && $follower->isTBA()) {
    Error::TBA_Overspecification();
  }
  return($leader, $follower);
}

# sub new_from_req {
#   my ($post_data_ref)=@_[0];
#   my %post_data=%$post_data_ref;
#   my $leader;
#   my $follower;
#   my $email_exists;
# 
#   General::log("Competitor::new_from_req...");
# 
#   if ($post_data{"leader_email_exists"} eq "false") {
#     $email_exists=0;
#   } else {
#     $email_exists=1;
#   }
#   $leader=Competitor->new($post_data{"leader_lname"},
# 			  $post_data{"leader_fname"},
# 			  $post_data{"leader_school"},
# 			  $post_data{"leader_email"},
# 			  $email_exists);
#   if ($post_data{"follower_email_exists"} eq "false") {
#     $email_exists=0;
#   } else {
#     $email_exists=1;
#   }
#   $follower=Competitor->new($post_data{"follower_lname"},
# 			    $post_data{"follower_fname"},
# 			    $post_data{"follower_school"},
# 			    $post_data{"follower_email"},
# 			    $email_exists);
#   return($leader, $follower);
# }
 
sub assignNumber {
  my $self=shift;
  my $number=$_[0];
  $self->{NUMBER}=$number;
  return($number);
}

# This actually makes a copy, which is not usually what we want.
#
# sub blessme {
#   my $proto=shift;
#   my $class=ref($proto) || $proto;
#   my $doppel=@_[0];
#   my $self={};
# 
#   $self->{LNAME}=$doppel->{LNAME};
#   $self->{FNAME}=$doppel->{FNAME};
#   $self->{SCHOOL}=$doppel->{SCHOOL};
#   $self->{EMAIL}=$doppel->{EMAIL};
#   $self->{EMAIL_EXISTS}=$doppel->{EMAIL_EXISTS};
#   $self->{NUMBER}=$doppel->{NUMBER};
#   $self->{TBA}=$doppel->{TBA};
# 
#   bless($self, $class);
#   return($self);
# }

sub doesEmailExist {
  my $self=shift;
  return($self->{EMAIL_EXISTS});
}

sub doesEmailExist2 {
  my $self=shift;
  if ($self->{EMAIL_EXISTS}) {
    return("true");
  } else {
    return("false");
  }    
}

sub equals {
  my $self=shift;
  my $that=$_[0];

#  General::log("Competitor equals: ".$self->{LNAME}.", ".$that->{LNAME}.
#	       "\n\t".$self->{FNAME}.", ".$that->{FNAME}.
#	       "\n\t".$self->{SCHOOL}.", ".$that->{SCHOOL}.
#	       "\n\t".$self->{EMAIL}.", ".$that->{EMAIL});

  return (($self->{LNAME} eq $that->{LNAME}) and
	  ($self->{FNAME} eq $that->{FNAME}) and
	  ($self->{SCHOOL} eq $that->{SCHOOL}));
}

sub getEmail {
  my $self=shift;
  return($self->{EMAIL});
}

sub getName {
  my $self=shift;
  return($self->{FNAME}." ".$self->{LNAME});
}

sub getFName {
  my $self=shift;
  return($self->{FNAME});
}

sub getLName {
  my $self=shift;
  return($self->{LNAME});
}

sub getNumber {
  my $self=shift;
  return($self->{NUMBER});
}

sub getSchool {
  my $self=shift;
  return($self->{SCHOOL});
}

sub getSchoolAbbrev {
  my $self=shift;
  $_=$self->{SCHOOL};
  if (/\(/) {
    ($crap, $_)=split('\(', $_);
    ($_, $crap)=split('\)', $_);
  } elsif ($_==-1) {
    return("<i>".Constants::get("HTML_MSG30")."</i>");
  }
  return($_);
}

sub getSortName {
  my $self=shift;
  return($self->{LNAME}." ".$self->{FNAME}." ".$self->{SCHOOL});
}

sub getTBA {
  my $self=shift;
  if ($self->{TBA}) {
#    return(1);
    return("true");
  } else {
#    return(0);
    return("false");
  }
}

# Obsolecent: Never used.  Made a dummy method awhile ago.
# sub hasFPartners {
#   return(0); # ;)
# }

sub isTBA {
  my $self=shift;
  return($self->{TBA});
}

# Of course, if you change this, you must change newFromDB.
sub write {
  my $self=shift;
  my $LOG=$_[0];

  print $LOG $self->{LNAME}."\n";
  print $LOG $self->{FNAME}."\n";
  print $LOG $self->{SCHOOL}."\n";
  print $LOG $self->{EMAIL}."\n";
  print $LOG $self->{EMAIL_EXISTS}."\n";
  print $LOG $self->{NUMBER}."\n";
  print $LOG $self->{TBA}."\n";
}

1; # so require is happy.
